###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
import obt.module

###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################

_descriptors = {}

def descriptor(architecture,osname):
  global _descriptors
  rval = None
  hash = "%s-%s" % (architecture,osname)
  if hash in _descriptors:
    rval = _descriptors[hash]
  else:  
    import obt.module
    import obt.path
    identifier = "%s-%s" % (architecture,osname)
    hi_name = obt.path.modules()/"sdk"
    hi_name = hi_name / ("%s.py"%identifier)
    the_module = obt.module.instance(identifier,hi_name)
    if the_module != None:
      hostident = obt.host.description().identifier
      if hasattr(the_module,"sdkinfo"):
        sdkinfo = the_module.sdkinfo()
        if hostident in sdkinfo.supports_host:
          _descriptors[hash] = sdkinfo
          rval = sdkinfo
    _descriptors[hash] = rval
  return rval

###############################################################################

class enuminterface:
  def __init__(self):
    self.subdir = "sdk"
  def tryAsModule(self,hostidentifier,item,pth):
    identifier = item.replace(".py","")
    m = obt.module.instance(identifier,pth)
    if hasattr(m,"sdkinfo"):
      sdki = m.sdkinfo()
      if hostidentifier in sdki.supports_host:
        return m
    else:
      return None

###############################################################################

def enumerate():
  iface = enuminterface()
  return obt.module.enumerate_simple(iface)



from yarl import URL
from obt import dep, path, pathtools, command

###############################################################################

class dspstretch(dep.StdProvider):
  VERSION = "c315378"
  name = "dspstretch"
  def __init__(self):
    build_dir = path.builds()/dspstretch.name
    super().__init__(dspstretch.name)
    
    pathtools.ensureDirectoryExists(path.includes()/"dspstretch")

    items = pathtools.patglob(self.source_root, "*.h")
    print(items)

    copy_commands = []
    for item in items:
      print(item)
      cmd = command.Command([
        "cp",str(item), str(path.includes()/"dspstretch")+"/"
        ])
      copy_commands.append(cmd)

    print(copy_commands)
    
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    self._builder._cleanbuildcommands = copy_commands
    self._builder._incrbuildcommands = copy_commands

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=dspstretch.name,
                             repospec="Signalsmith-Audio/signalsmith-stretch",
                             revision=dspstretch.VERSION,
                             recursive=True)

  #######################################################################

  def areRequiredSourceFilesPresent(self):
    print(self.source_root)
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"dspstretch"/"signalsmith-stretch.h").exists()



#!/usr/bin/env python3

import os, sys, subprocess, argparse

from obt import command, path, pathtools, sdk, dep

parser = argparse.ArgumentParser(description='obt.build dep builder')
parser.add_argument('--gencert', action="store_true", help='generate ios certificate' )
parser.add_argument('--buildios', action="store_true", help='generate ios certificate' )

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

args = parser.parse_args()

##############################################

prefix = path.subspace_root()/"ios"
pathtools.ensureDirectoryExists(prefix)
pathtools.ensureDirectoryExists(prefix/"builds")
pathtools.ensureDirectoryExists(prefix/"lib")
pathtools.ensureDirectoryExists(prefix/"bin")
pathtools.ensureDirectoryExists(prefix/"conan")
os.chdir(prefix)

##############################################

CERTNAME = "APHIDSYSC"

def generate_csr(output_path, certificate_name):
    """Generate a Certificate Signing Request (CSR) and add key to keychain."""
    try:
        # Use openssl to generate a CSR and key
        subprocess.run(["openssl", "req", "-new", "-newkey", "rsa:2048", "-nodes", #
                        "-keyout", f"{output_path}/{certificate_name}.key", # 
                        "-out", f"{output_path}/{certificate_name}.csr"], #
                        check=True)
        # Add the key to the keychain
        subprocess.run(["security", "import", f"{output_path}/{certificate_name}.key", "-k", "login.keychain", "-T", "/usr/bin/codesign"],
                        check=True)
        print("CSR generated and key added to keychain successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error generating CSR and adding key to keychain: {e}")
        raise

if args.gencert:
  print("Generating iOS certificate...")
  generate_csr(str(prefix), CERTNAME )
  print("Certificate generated successfully.")
  print("Now, upload the certificate signing request to the Apple Developer portal to create a certificate.")
  sys.exit(0)


##############################################

TEMP_PATH = path.temp()
IOS_SDK = sdk.descriptor("aarch64","ios")
SDK_DIR = IOS_SDK._sdkdir
SDK_VER = IOS_SDK._sdkver
the_environ = {
  "OBT_SUBSPACE_BUILD_DIR": prefix/"builds",
  "OBT_SUBSPACE_LIB_DIR": prefix/"lib",
  "OBT_SUBSPACE_DIR": prefix,
  "OBT_SUBSPACE_BIN_DIR": prefix/"bin",
  "IOS_PREFIX": prefix,
  "IOS_SDK_DIR": SDK_DIR,
  "IOS_SDK_VER": SDK_VER,
  "IOS_CLANG_PATH": IOS_SDK._clang_path,
  "IOS_CLANGPP_PATH": IOS_SDK._clangpp_path,
  "OBT_SUBSPACE": "ios",
  #"CMAKE_TOOLCHAIN_FILE": prefix/"ios.toolchain.cmake",
  #"OBT_SUBSPACE_PROMPT": self._gen_sysprompt(),
  "OBT_TARGET": "aarch64-ios",
  "CONAN_HOME": prefix/"conan",
  "CONAN_USER_HOME": prefix/"conan",
  "CONAN_CACHE_DIR": "~/.obt-global/conan2"
}     

##############################################
# generate cmake toolchain
##############################################

cmake_tc = """
set(IOS_SDK_PATH $ENV{IOS_SDK_DIR})
set(CMAKE_SYSTEM_NAME iOS)

# Specify the architectures to build for (e.g., arm64 for actual devices)
set(CMAKE_OSX_ARCHITECTURES arm64)

# Specify the minimum iOS deployment target
set(CMAKE_OSX_DEPLOYMENT_TARGET $ENV{IOS_SDK_VER})

# Specify the path to the compiler
set(CMAKE_C_COMPILER $ENV{IOS_CLANG_PATH})
set(CMAKE_CXX_COMPILER $ENV{IOS_CLANGPP_PATH})
set(CMAKE_INSTALL_PREFIX $ENV{OBT_SUBSPACE_BUILD_DIR})
"""

tc_output = prefix/"ios.toolchain.cmake"
with open(tc_output,"w") as f:
  f.write(cmake_tc)

##############################################
# generate conanfile
##############################################

# missing 
# glfw
# lexertl14
# parsertl14
# easyprof 
# rtmidi 
# cpppeglib 
# klein 
# portaudio 

# not working
#libigl/2.3.0
#openimageio/2.5.6
#rapidjson/cci.20230929 <incompatible with assimp/5.2.2>
#openblas/0.3.26 

lexertl = dep.instance("lexertl14")
print(lexertl)
#print(dir(lexertl))
print(lexertl._conanfile)

lexertl14_dir = prefix / "conan" / "lexertl14"
lexertl14_dir.mkdir(parents=True, exist_ok=True)  # Ensure the directory exists
lrtl_output = lexertl14_dir / "conanfile.py"
with open(lrtl_output, "w") as f:
    f.write(lexertl._conanfile)  # Assuming lexertl._conanfile is the recipe text
os.chdir(str(lexertl14_dir))
#conan export . lexertl14/tweakoz-obt@user/channel
command.run(["conan","export",".",
             "--user=user",
             "--channel=channel"], #
             environment=the_environ,do_log=True ) 

conanfile = """
[requires]
zlib/1.2.11
boost/1.81.0
assimp/5.2.2
ptex/2.4.2
openexr/3.2.1
rapidjson/cci.20220822
eigen/3.4.0
glm/cci.20230113
bullet3/3.25
zeromq/4.3.5
cppzmq/4.10.0
sigslot/1.2.2 
moltenvk/1.2.2
lexertl14/tweakoz-obt

[generators]
CMakeDeps
CMakeToolchain
"""

cf_output = prefix/"conanfile.txt"
with open(cf_output,"w") as f:
  f.write(conanfile)

##############################################
# generate conan profile for ios
##############################################

conan_host_profile = """
[settings]
os=iOS
os.version=17.0
arch=armv8
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
os.sdk=iphoneos
"""

cp_output = prefix/"ios.host.profile"
with open(cp_output,"w") as f:
  f.write(conan_host_profile)

##############################################

conan_build_profile = """
[settings]
os=Macos
arch=armv8  
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
"""

cp_output = prefix/"ios.build.profile"
with open(cp_output,"w") as f:
  f.write(conan_build_profile)

##############################################
  
cmd =  ["conan","install","."]
cmd += [f"--profile:host={prefix}/ios.host.profile"]
cmd += [f"--profile:build={prefix}/ios.build.profile"]
cmd += ["--build=missing"]

command.run(cmd,environment=the_environ,do_log=True)

##############################################

cmake_lists_content = """
cmake_minimum_required(VERSION 3.15)
project(ios_minimal_app LANGUAGES CXX OBJCXX)
set(CMAKE_OSX_SYSROOT %s CACHE PATH "The iOS SDK sysroot")

#include(${CMAKE_BINARY_DIR}/conan_toolchain.cmake)
include(${CMAKE_CURRENT_SOURCE_DIR}/ios.toolchain.cmake)

find_package(ZLIB REQUIRED)

add_executable(${PROJECT_NAME} main.mm)
set_target_properties(${PROJECT_NAME} PROPERTIES
    XCODE_ATTRIBUTE_PRODUCT_BUNDLE_IDENTIFIER "com.example.minimalapp"
    XCODE_ATTRIBUTE_DEVELOPMENT_TEAM "YOUR_TEAM_ID"
    MACOSX_BUNDLE_GUI_IDENTIFIER "com.example.minimalapp"
    MACOSX_BUNDLE_INFO_PLIST ${CMAKE_CURRENT_SOURCE_DIR}/Info.plist
)
target_link_libraries(${PROJECT_NAME} PRIVATE ZLIB::ZLIB)
""" % SDK_DIR

with open(prefix / "CMakeLists.txt", "w") as f:
    f.write(cmake_lists_content)

##############################################

info_plist_content = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>CFBundleDisplayName</key>
    <string>MyApp</string>
    <key>CFBundleExecutable</key>
    <string>MyApp</string>
    <key>CFBundleIdentifier</key>
    <string>com.aphidsystems.myapp</string>
    <key>CFBundleInfoDictionaryVersion</key>
    <string>6.0</string>
    <key>CFBundleName</key>
    <string>MyApp</string>
    <key>CFBundlePackageType</key>
    <string>APPL</string>
    <key>CFBundleShortVersionString</key>
    <string>1.0</string>
    <key>CFBundleVersion</key>
    <string>1</string>
    <key>LSRequiresIPhoneOS</key>
    <true/>
    <key>UILaunchStoryboardName</key>
    <string>LaunchScreen</string>
    <key>UIRequiredDeviceCapabilities</key>
    <array>
        <string>armv7</string>
    </array>
    <key>UISupportedInterfaceOrientations</key>
    <array>
        <string>UIInterfaceOrientationPortrait</string>
    </array>
</dict>
</plist>
"""

info_plist_path = prefix / "Info.plist"
with open(info_plist_path, "w") as f:
    f.write(info_plist_content)

##############################################

# main.mm
objc_plus_plus_source = """
#include <iostream>
#include <zlib.h>
#include <string>

int main(int argc, const char * argv[]) {
    std::string originalStr = "Hello, Conan and iOS World!";
    uLongf compressedDataSize = compressBound(originalStr.size());
    Bytef *compressedData = (Bytef*)malloc(compressedDataSize);

    if (compress(compressedData, &compressedDataSize, (Bytef*)originalStr.data(), originalStr.size()) == Z_OK) {
        std::cout << "Original string: " << originalStr << "\\n";
        std::cout << "Compressed size: " << compressedDataSize << "\\n";
    } else {
        std::cout << "Compression failed!" << "\\n";
    }

    free(compressedData);
    return 0;
}
"""

# Write the source to a file
app_source_file = prefix / "main.mm"
with open(app_source_file, "w") as f:
    f.write(objc_plus_plus_source)

pathtools.ensureDirectoryExists(prefix/".build")
os.chdir(prefix/".build")

the_environ["VERBOSE"] = "1"

print( "############## begin CMakelists.txt ##############")
print(cmake_lists_content)
print( "############## end CMakelists.txt ##############")

print( "############## begin envdump ##############")

for item in the_environ:
  print(f"{item}={the_environ[item]}")

print( "############## end envdump ##############")

command.run(["cmake", "..", "-DCMAKE_BUILD_TYPE=Release"], environment=the_environ, do_log=True)
#command.run(["cmake", ".", "-DCMAKE_BUILD_TYPE=Release", environment=the_environ, do_log=True)

# Build the project with CMake
command.run(["cmake", "--build", "."], environment=the_environ, do_log=True)

# Create app bundle structure
app_bundle_dir = prefix / "MyApp.app"
os.makedirs(app_bundle_dir, exist_ok=True)

# Copy executable
executable_src = prefix / ".build/ios_minimal_app"  # Change this to the actual path of your executable
executable_dest = app_bundle_dir / "MyApp"  # Change "MyApp" to your app's name
pathtools.copyfile(executable_src, executable_dest)

# Copy Info.plist

# Add any other necessary resources (images, storyboards, etc.) to the Resources directory
pathtools.copyfile(prefix / "Info.plist", app_bundle_dir / "Info.plist")

# Code sign the app bundle (you may need to adjust this based on your code signing configuration)
command.run(["codesign", "-s", CERTNAME,  "--deep", app_bundle_dir], do_log=True)

command.run(["ideviceinstaller","-i",prefix/"MyApp.app"], do_log=True)


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class lexertl14(dep.StdProvider):
  name = "lexertl14"
  def __init__(self):
    super().__init__(lexertl14.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=lexertl14.name,
                             repospec="tweakoz/lexertl14",
                             revision="toz-oct16",
                             recursive=False)
 #######################################################################
  @property
  def _conanfile(self):
    build_src = self.build_src/"include"
    X = """
from conan import ConanFile
from conan.tools.cmake import CMake, cmake_layout
class LexerTL14Conan(ConanFile):
  name = "lexertl14"
  version = "tweakoz-obt"
  license = "<License here>"
  author = "<Your Name> <Your Email>"
  url = "https://github.com/tweakoz/lexertl14"
  description = "<Description of lexertl14 here>"
  topics = ("<Put some tag here>", "<Put another tag here>")
  settings = "os", "compiler", "build_type", "arch"
  exports_sources = ["include/*"]  # Adjust the pattern to match your source directory structure

  def layout(self):
    cmake_layout(self)

  def build(self):
    cmake = CMake(self)
    cmake.configure()
    cmake.build()

  def package(self):
    self.copy("*", dst="include", src="%s")  # Adjust 'src' to the correct source subdirectory if needed

  def package_info(self):
    self.cpp_info.libs = ["lexertl14"]
      """ % build_src

    return X
  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"lexertl"/"char_traits.hpp").exists()



import obt.xcode 
import obt.command 
import obt.env
import obt.path 
import os
import subprocess

class _iossdk_private:
  def __init__(self):
    self._xcodesdkstr = obt.command.capture([
      "xcodebuild",
      "-version",
      "-sdk"])
    self.has_iossdk15_2 = self._xcodesdkstr.find("(iphoneos15.2)")>0
    self.has_iossdk15_5 = self._xcodesdkstr.find("(iphoneos15.5)")>0
    self.has_iossdk17_0 = self._xcodesdkstr.find("(iphoneos17.0)")>0
    print("has_iossdk15_2<%s>"%self.has_iossdk15_2)
    print("has_iossdk15_5<%s>"%self.has_iossdk15_5)
    print("has_iossdk17_0<%s>"%self.has_iossdk17_0)
    if self.has_iossdk15_2:
      self._iossdkver = "15.2"
      self._iossdkstr = obt.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "iphoneos15.2",
        "Path"],do_log=False).splitlines()
      #obt.env.set("OBT_IOS_SDK","iphoneos15.2")
      #obt.env.set("OBT_IOS_SDK_DIR",self._iossdkstr)
    elif self.has_iossdk15_5:
      self._iossdkver = "15.5"
      self._iossdkstr = obt.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "iphoneos15.5",
        "Path"],do_log=False).splitlines()
      #obt.env.set("OBT_IOS_SDK","iphoneos15.5")
      #obt.env.set("OBT_IOS_SDK_DIR",self._iossdkstr)
    elif self.has_iossdk17_0:
      self._iossdkstr = obt.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "iphoneos17.0",
        "Path"],do_log=False).splitlines()[0]
      self._iossdkver = "17.0"
      #obt.env.set("OBT_IOS_SDK","iphoneos17.0")
      #obt.env.set("OBT_IOS_SDK_DIR",a)
    try:
      # Use xcrun to find the path to clang for the current SDK
      clang_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang"
      ], text=True).strip()
      clangpp_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang++"
      ], text=True).strip()

      # Set environment variable for the compiler path
      self._clang_path = clang_path
      self._clangpp_path = clangpp_path

    except subprocess.CalledProcessError as e:
      print(f"Error executing command: {e}")
    except Exception as e:
      print(f"An unexpected error occurred: {e}")


class sdkinfo:
  #################################################
  # env_init invoked once at container startup time
  #################################################

  #############################################
  def __init__(self):
    self.identifier = "aarch64-ios"
    self.architecture = "aarch64"
    self.os = "ios"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-macos","aarch64-macos"]
  #############################################
  @property
  def sdkdir(self):
    return obt.path.Path(os.environ["OBT_IOS_SDK_DIR"])
  @property
  def _sdkdir(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._iossdkstr
  @property
  def _sdkver(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._iossdkver
  @property
  def _clang_path(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._clang_path
  @property
  def _clangpp_path(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._clang_path
  #############################################
  def misc(self):
    return {
      "sdkdir": self.sdkdir,
      "_sdkdir": self._sdkdir,
      "_sdkver": self._sdkver,
      "_clang_path": self._clang_path,
      "_clangpp_path": self._clangpp_path
    }




###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys

###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

target_arch = None 
target_os = None

if "OBT_TARGET" not in os.environ:
  SYSTEM = platform.system()
  ##############
  IsOsx = (SYSTEM=="Darwin")
  IsDarwin = (SYSTEM=="Darwin")
  IsIrix = (SYSTEM=="IRIX64")
  IsLinux = (SYSTEM=="Linux")
  ##############
  IsX86_64 = platform.machine()=="x86_64"
  IsAARCH64 = (platform.machine()=="aarch64") or (platform.machine()=="arm64")
  IsX86_32 = platform.machine()=="i686"
  IsAppleSilicon = IsAARCH64 and IsDarwin
  ##############
  if IsX86_32:
    machine = "x86_32"
  elif IsAARCH64:
    machine = "aarch64"
  else:
    machine = "x86_64"
  target_arch = machine
  if IsOsx:
    target_os = "macos"
  elif IsLinux:
    target_os = "linux"
            
else:
  TARGET_STR = os.environ.get("OBT_TARGET")
  target_arch = TARGET_STR.split("-")[0]
  target_os = TARGET_STR.split("-")[1]

is_linux = (target_os=="linux")
is_macos = (target_os=="macos")
is_windows = (target_os=="windows")
is_android = (target_os=="android")
is_ios = (target_os=="ios")

###############################################################################

def descriptor(architecture,osname):
  import obt.module
  import obt.path
  identifier = "%s-%s" % (architecture,osname)
  hi_name = obt.path.modules()/"target"
  hi_name = hi_name / ("%s.py"%identifier)
  the_module = obt.module.instance(identifier,hi_name)
  if the_module != None:
    return the_module.targetinfo()

###############################################################################

class enuminterface:
  def __init__(self):
    self.subdir = "target"
  def tryAsModule(self,item,pth):
    return None

###############################################################################

def enumerate():
  iface = enuminterface()
  return obt.module.enumerate_simple(iface)
